package App::IdiotBox::Importer;

use strict;
use warnings FATAL => 'all';
use Cwd;
use IO::All;
use ExtUtils::MakeMaker qw(prompt);
use File::Spec::Functions qw(catfile catdir);
use POSIX qw(strftime);

use Data::ICal;
use Text::LevenshteinXS 'distance';

sub log_info (&) { print $_[0]->(), "\n"; }

sub run {
  my ($class, $ib) = @_;
  my @buckets = $ib->buckets->flatten;
  my %bucket_by_slug;
  log_info { "Available buckets to import into:" };
  foreach my $idx (0 .. $#buckets) {
    my $bucket = $buckets[$idx];
    $bucket_by_slug{$bucket->slug} = $bucket;
    log_info { "(${idx}) ${\$bucket->slug} : ${\$bucket->name}" };
  }

  my $bucket;
    
  CHOOSE: {
    my $choice = prompt("Which bucket to import into (by number or slug) ?");
    if ($choice =~ /^\d+$/) {
      $bucket = $buckets[$choice];
    } else {
      $bucket = $bucket_by_slug{$choice};
    }
    unless ($bucket) {
      log_info {
         "No bucket for ${choice} - valid options are 0 to ${\$#buckets}"
         ." or slug (e.g. ${\$buckets[0]->slug})"
       };
       redo CHOOSE;
    }
  }

  my $ann = $ib->recent_announcements->add(bless({
    bucket => $bucket,
    made_at => strftime("%Y-%m-%d %H:%M:%S",localtime),
  }, 'App::IdiotBox::Announcement'));

  log_info { "Created new announcement, id ".$ann->id };

  my $video_files = $class->video_files_from_dir(my $source_dir = cwd);

  my %videos;

  foreach my $video_file (keys %{$video_files}) {

    log_info { "Processing file ${video_file}" };
    my @parts = split(/[- ]+/, $video_file);
    my @options;
    foreach my $idx (1 .. $#parts) {
      my @opt = @{$options[$idx] = [
        join(' ', @parts[0..$idx-1]),
        join(' ', @parts[$idx..$#parts]),
      ]};
      log_info { "(${idx}) ".join(' / ', @opt) };
    }
    my $info;
    CHOICE: {
      my $choice = prompt(
        'What author is this for (enter number for pre-selected combination) ?',
        2
      );
      if ($choice =~ /^\d+$/) {
        @{$info}{qw(author name)} = @{$options[$choice] || redo CHOICE};
      } else {
        $info->{author} = $choice;
      }
    }
    $info->{name} = prompt('What is the name of this talk?', $info->{name});
    (my $slug = lc $info->{name}) =~ s/ /-/g;
    $info->{slug} = prompt('What is the slug for this talk?', $slug);
    my $act_data = {};
    $act_data = get_act_talk_data($info->{name}, $bucket->{act_url})
      if $bucket->{act_url};
    if(exists $act_data->{talk_id}) {
        $info->{act_talk_id} = $act_data->{talk_id};
        $info->{details} = $act_data->{description};
    }
        
    $info->{details} = prompt('Enter a talk description', $info->{details});
    $info->{bucket} = $bucket;
    $info->{announcement} = $ann;
    $videos{$video_file} = bless($info, 'App::IdiotBox::Video');
  }
  foreach my $video_file (keys %videos) {
    my $video = $videos{$video_file};
    my $target_dir = catdir($ib->config->{base_dir}, $video->url_path);
    io($target_dir)->mkpath;
    log_info { "Copying video files to ${target_dir}"};
    foreach my $ext (@{$video_files->{$video_file}}) {
      no warnings 'void';
      io(catfile($target_dir, "${\$video->file_name}.${ext}"))
        < io(catfile($source_dir, "${video_file}.${ext}"));
    }
  }
  
  $bucket->videos->add($_) for values %videos;
}

## Bug, this doesn't cope with files named foo.bar.5.23.mkv etc (multiple periods)
sub video_files_from_dir {
  my ($class, $dir) = @_;
  my %videos;
  foreach my $file (io($dir)->all_files) {
    $file->filename =~ /^([^\.]+)\.([^\.]+)$/ or next;
    push(@{$videos{$1}||=[]}, $2);
  }
  \%videos;
}

## Parse the act timetable.ics to find talk data and description
## (best guess based on talk name)
sub get_act_talk_data {
  my ($talk_name, $base_url) = @_;

  my $act_url = URI->new($base_url . '/timetable.ics');
  my $timetable < io($act_url);

  my $act_schedule = Data::ICal->new(data => $timetable)
    or die "Can't parse timetable.ics: $!";

  my $target = "Things I learned from users";

  my $mangled_target = uc $target;
  $mangled_target =~ s/\W//g;

  my @possibles;

  for my $entry (@{$act_schedule->entries}) {
    my $title = $entry->property('SUMMARY')->[0]->decoded_value;

    my $description = $entry->property('DESCRIPTION')->[0]->decoded_value;

    my $eid = $entry->property('URL')->[0]->decoded_value;
    $eid =~ s!/$!!;
    $eid =~ m!.*/(.*?)$! or die "Couldn't find last component in $eid";
    $eid = $1;

    my $mangled_title = uc $title;
    $mangled_title =~ s/\W//g;

    my $score = (length($mangled_title) - distance($mangled_title, $mangled_target))/length($mangled_title);
    # print "Score for $mangled_title vs $mangled_target = $score\n";

    push @possibles, {
                      score => $score,
                      title => $title,
                      description => $description,
                      event_id => $eid
                     };
  }

  @possibles = sort {$b->{score} <=> $a->{score}} @possibles;


  print STDERR "ye2010/$target/ http://conferences.yapceurope.org/ye2010/talk/$possibles[0]{event_id}\n";

  return $possibles[0];

}


1;
